INCLUDE "hardware.inc"
INCLUDE "sound.inc"

DEF SOUND_END EQU $FF
DEF CHANGE_PRIORITY EQU $FE
DEF PAN_CH1 EQU $FD
DEF PAN_CH2 EQU $FC
DEF PAN_CH3 EQU $FB
DEF PAN_CH4 EQU $FA
DEF SET_MUSIC_MUTE EQU $F9

DEF MUTE_CH1 EQU 0 << 4
DEF MUTE_CH2 EQU 1 << 4
DEF MUTE_CH3 EQU 2 << 4
DEF MUTE_CH4 EQU 3 << 4
DEF MUSIC_MUTE EQU 1
DEF MUSIC_UNMUTE EQU 0

DEF AUDHIGH_NO_RESTART EQU 0 ; missing from hardware.inc (counterpart to AUDHIGH_RESTART)

; --- Channel 2 ---

; \1 = Duty (use the AUDLEN_DUTY definitions from hardware.inc)
; \2 = Sound Length (0-63)
; \3 = Frame Wait (0-255)
MACRO CH2_LENGTH_DUTY
    DB LOW(rNR21), \1 | \2, \3
ENDM

; \1 = Initial Volume (0-$F)
; \2 = Envelope Direction (use the AUDENV definitions from hardware.inc)
; \3 = Number of envelope sweep (0-7)
; \4 = Frame Wait (0-255)
MACRO CH2_VOLENV
    DB LOW(rNR22), (\1 << 4) | \2 | \3, \4
ENDM

; \1 = Frequency (use the NOTE definitions)
; \2 = Stop sound after length expires (use the AUDHIGH_LENGTH defs from hardware.inc)
; \3 = If sound should be restarted (use the AUDHIGH defs)
; \4 = Number of frames to wait after effect (0-255)
MACRO CH2_FREQ
    DB LOW(rNR23), LOW(\1), 0
    DB LOW(rNR24), HIGH(\1) | \2 | \3, \4
ENDM

; --- Channel 4 ---

; \1 = Sound Length (0-63)
; \2 = Frame Wait (0-255)
MACRO CH4_LENGTH
    DB LOW(rNR41), \1, \2
ENDM

; \1 = Initial Volume (0-$F)
; \2 = Envelope Direction (use the AUDENV definitions from hardware.inc)
; \3 = Number of envelope sweep (0-7)
; \4 = Frame Wait (0-255)
MACRO CH4_VOLENV
    DB LOW(rNR42), (\1 << 4) | \2 | \3, \4
ENDM

; \1 = Counter Width (use the AUD4POLY defs)
; \2 = Shift Clock Frequency (0-15)
; \3 = Frequency Dividing Ratio (0-7)
; \4 = Frame Wait (0-255)
MACRO CH4_POLYCT
    DB LOW(rNR43), \1 | (\2 << 4) | \3, \4
ENDM

; \1 = Stop sound after length expires (use the AUDHIGH_LENGTH defs from hardware.inc)
; \2 = If sound should be restarted (use the AUDHIGH defs)
; \3 = Frame Wait (0-255)
MACRO CH4_RESTART
    DB LOW(rNR44), \1 | \2, \3
ENDM

SECTION FRAGMENT "Sound FX", ROMX

; https://daid.github.io/gbsfx-studio/

FX_ShortCrash:: ; (CH4) Played for small "bumps" between cars
    DB 1 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $6, AUDENV_DOWN, 1, 0
    CH4_POLYCT AUD4POLY_15STEP, $8, 0, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_CarExplode:: ; (CH4) Played for car explosions
    DB 1 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $D, AUDENV_DOWN, 2, 0
    CH4_POLYCT AUD4POLY_15STEP, $9, 0, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 20
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_PlayerMissile:: ; (CH4) When player fires a missile
    DB 1 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $C, AUDENV_DOWN, 3, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 0
    CH4_POLYCT AUD4POLY_15STEP, $4, 0, 1
    CH4_POLYCT AUD4POLY_15STEP, $5, 0, 1
    CH4_POLYCT AUD4POLY_15STEP, $6, 1, 29
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_EnemyMissile:: ; (CH4) When an enemy fires a missile
    DB 1 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $C, AUDENV_DOWN, 3, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 0
    CH4_POLYCT AUD4POLY_15STEP, $4, 0, 1
    CH4_POLYCT AUD4POLY_15STEP, $5, 0, 1
    CH4_POLYCT AUD4POLY_15STEP, $6, 2, 29
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_HeliExplode:: ; (CH4) Played for helicopter explosions
    DB 1 ; FX channel
    DB 2 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $E, AUDENV_DOWN, 3, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 0
    CH4_POLYCT AUD4POLY_15STEP, $4, 1, 1
    CH4_POLYCT AUD4POLY_15STEP, $6, 1, 1
    CH4_POLYCT AUD4POLY_15STEP, $7, 2, 35
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_TruckDropRock:: ; (CH4) Played when the truck uses it's special to drop a rock
    DB 1 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $D, AUDENV_DOWN, 2, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 0
    CH4_POLYCT AUD4POLY_15STEP, $4, 3, 20
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_TimeCarBlip:: ; (CH4) Played when the time car activates it's special
    DB 1 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_MUTE, 0
    DB PAN_CH4, AUDTERM_4_LEFT | AUDTERM_4_RIGHT, 0
    CH4_VOLENV $C, AUDENV_DOWN, 3, 0
    CH4_POLYCT AUD4POLY_7STEP, $2, 7, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 2
    CH4_POLYCT AUD4POLY_15STEP, $1, 6, 0
    CH4_RESTART AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 20
    DB SET_MUSIC_MUTE, MUTE_CH4 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_WarningBeep:: ; (CH2) Played when the warning sign flashes
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $E, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_D_4, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 7
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_PlayerMissileCharged:: ; (CH2) Played when the player's missile bar is fully charged
    DB 0 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $7, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_Cs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    CH2_FREQ NOTE_F_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_PlayerSpecialCharged:: ; (CH2) Played when the player's special bar is fully charged
    DB 0 ; FX channel
    DB 1 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $7, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_Ds5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    CH2_FREQ NOTE_G_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_MenuBip:: ; (CH2) The small bip when moving between menu items
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_12_5, 0, 0
    CH2_VOLENV $8, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_A_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_CarSelect:: ; (CH2) When a car is selected in the garage
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $A, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_D_4, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 5
    CH2_FREQ NOTE_E_4, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 6
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_CarBuy:: ; (CH2) When a car is bought / upgraded in the garage
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $A, AUDENV_DOWN, 2, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 2
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_CarFailBuy:: ; (CH2) When you try to buy / upgrade a car but don't have enough money
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $A, AUDENV_DOWN, 2, 0
    CH2_FREQ NOTE_Ds4, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 2
    CH2_FREQ NOTE_Fs4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_As4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Ds4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_As4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Ds4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_As4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Ds4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_Fs4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    CH2_FREQ NOTE_As4, AUDHIGH_LENGTH_OFF, AUDHIGH_NO_RESTART, 2
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_Pause:: ; (CH2) Played when the pause menu is opened
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $F, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_C_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_FREQ NOTE_D_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_FREQ NOTE_F_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 4
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_Unpause:: ; (CH2) Played when the pause menu is closed
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_50, 0, 0
    CH2_VOLENV $F, AUDENV_DOWN, 1, 0
    CH2_FREQ NOTE_F_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_FREQ NOTE_D_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_FREQ NOTE_C_3, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 4
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END

FX_TitleScreenStart:: ; (CH2) Played when pressing "Start" on the title screen
    DB 0 ; FX channel
    DB 0 ; Starting priority byte
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_MUTE, 0
    DB PAN_CH2, AUDTERM_2_LEFT | AUDTERM_2_RIGHT, 0
    CH2_LENGTH_DUTY AUDLEN_DUTY_75, 0, 0
    CH2_VOLENV $F, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $E, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $D, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $C, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $B, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $A, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $9, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $8, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $7, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $6, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $5, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $4, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $3, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_D_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $2, AUDENV_DOWN, 0, 0
    CH2_FREQ NOTE_Fs5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    CH2_VOLENV $1, AUDENV_DOWN, 2, 0
    CH2_FREQ NOTE_A_5, AUDHIGH_LENGTH_OFF, AUDHIGH_RESTART, 3
    DB SET_MUSIC_MUTE, MUTE_CH2 | MUSIC_UNMUTE, 0
    DB SOUND_END